*   Colorset.prg 
*   program to change screen colors for dBASE IV.

*                   Copyright (c) 1989 Jay Parsons.

**        External files and routines called upon    

*    Binary modules, which must be present on a
*    directory in the dBASE path

*    VINFO.BIN   obtains video display information
*    PEEK.BIN    peeks information from memory address
*    SHOWIT.BIN  writes to screen without changing
*                  dBASE colors
*                (needed only if Speedup option = .T.)
    
*    Functions and procedures, which must be in the
*    active procedure file or must be coded into this
*    program file

*    dectohex(n)   converts number n to hex string
*    dosfindf(c)   finds file c in current directory
*                      or DOS path
*    make_dbf(c,f,x) creates .dbf file c with fields
*                    f[1]-f[x], where f is an array
*                       of dimension [x,4]
*      (needed only if (Editfile) created, see below)

*    cchange      changes colors to values in
*                    "C_" variables

*    Data and other dBASE files 

*    None required.  However, the edit file (Editfile)
*    will be created if not existent, using make_dbf,
*    in either of two cases:

*    1)  No .dbf files exist in the current directory.
*    2)  The Reconfig option is .T. and reads from or
*        writes to Config.db are requested.

**        Statuses changed by program 

*    If (Editfile) is created, it will displace the
*    file, if any, that was active in the current work
*    area before the program was run, and will not
*    restore it.  Its creation will also nullify the
*    printer set with the SET PRINTER TO command,
*    leaving no printer set.

*    The colors set by this program will be in effect,
*    unless cancelled within the program, for all
*    screen output after the program terminates, but
*    will not necessarily appear at once since the
*    program restores the screen as it existed before
*    this program commenced.

**        Default definitions (May be changed at will)          

*    Columns for display -- range is 0 - 79

Labelclm=6          && column of labels
Showclm=24          && column of color blocks
Fgclm=42            && column of foreground options
Bgclm=60            && column of background options

*    Keys to navigate and messages, which should be
*    changed if keys are.  Definitions are inkey()
*    values.

Dn=3                && PgDn to move down screen
Up=18               && PgUp to move up screen
Ucolor=24           && down arrow to increase color
Dcolor=5            && up arrow to decrease color 
L=19                && left arrow to move left
R=4                 && and right to move right
Q=13                && Enter to quit

Navmess1="Use PgUp, PgDn and right and left arrows ";
  +"to move around screen."
Navmess2="Use up and down arrows to cycle through ";
  +"colors. "

*    Seconds for copyright notice to display

Banner=2

*    Extension for color-save files.
*    Should not be .MEM or user may overwrite
*    a .MEM file containing important variables
*    unrelated to colors.

Memext="MCM"

*         Developer's option for method of setting
*         default colors.

*    If Reconfig is .T., defaults will be read from
*    and written to the Config.db file in the current
*    directory if any, otherwise the one first on the
*    DOS path.

*    If Reconfig is .F., defaults will be read from
*    and written to a memory file.  It is assumed that
*    a command to RESTORE FROM this file ADDITIVE, and
*    the code of the procedure cchange or a call to
*    it, will be placed in whatever procedure is
*    executed on startup.

Reconfig=.F.

*    File for editing Config.db.  Must have a
*    character field of sufficient length to hold
*    each line of Config.db, which field must be the
*    first field of the structure and should
*    preferably be the only field.

Editfile="Confedit.dbf"   && current directory assumed

*    Memory file for save/restore of default colors

IF .NOT. Reconfig
  Dflfile="Dflcolor.m$m"  && current directory assumed
ENDIF

*    Developer's option for method of implementing
*    color selections.

*    If Speedup is .T., the file Showit.bin is
*    required, and will write color selections
*    directly to the screen using the BIOS.
*    The colors known to dBASE will not be changed
*    until selection is complete.

*    If Speedup is .F., the SET COLOR command will
*    be used on each selection, which will be
*    excruciatingly slow on some systems.

Speedup=.T.

*    Developer's option for dealing with overscan.

*    If Setborder is .T., the overscan color will
*    be set to whatever color is selected for the
*    background of the NORMAL color area, and saved
*    accordingly.

*    If Setborder is .F., the overscan color and
*    any Config.db command setting it will be left
*    alone.

Setborder=.T.

**        Other definitions -- changes may require
**        changing dependent code.

Block=16            && color block length <even #>
Savemess="  Save and exit " && length=Block
Canmess= "     Cancel     " &&    ditto
Savemes2="Press ENTER to save colors selected"
Canmes2="Press ENTER to cancel changes and quit"
Saveclm=Fgclm       && other choice is Bgclm

**        Popup definitions

*    Good practice requires different first
*    characters for each prompt, so that pressing
*    that key will activate the intended selection.

*     Colors to start from       

DEFINE POPUP Init FROM 10,18 TO 14,61
IF Reconfig
  DEFINE BAR 1 OF Init ;
     PROMPT "Default colors from CONFIG.DB"
ELSE
  IF FILE(Dflfile)
    DEFINE BAR 1 OF Init ;
       PROMPT "Default colors from "+Dflfile
  ELSE
    DEFINE BAR 1 OF Init ;
       PROMPT "<no default colors saved>" SKIP
  ENDIF  
ENDIF
DEFINE BAR 2 OF Init PROMPT "Memory - ";
  +"colors in a memory (."+Memext+") file"
DEFINE BAR 3 OF Init ;
  PROMPT "Present - colors presently in effect"
ON SELECTION POPUP Init DO init_pop

*     Saved color memory files

DEFINE POPUP mems FROM 3,55 ;
  PROMPT FILES LIKE *.&Memext MESSAGE ;
  "Please select the save file or press Esc"

*     Save options 

DEFINE POPUP csave FROM 5,15 TO 12,64
DEFINE BAR 1 of Csave PROMPT ;
  "Defaults - Save colors in " ;
  +iif(Reconfig,"CONFIG.DB",Dflfile)
DEFINE BAR 2 OF Csave PROMPT ;
  "Memory - Save colors to an ";
  +"existing (."+Memext+") file"
DEFINE BAR 3 OF Csave PROMPT ;
  "New - Save colors to a new memory ";
  +"(."+Memext+") file"
DEFINE BAR 4 OF Csave PROMPT ;
  "Keep selected colors for this session only"
DEFINE BAR 5 OF Csave PROMPT ;
  "Show selected colors and permit more changes"
DEFINE BAR 6 OF Csave PROMPT ;
  "Cancel changes - restore original colors"
ON SELECTION POPUP Csave DO cs_pop

*    Note that the rows on which items are displayed
*    on the Selection screen are used for arithmetic
*    calculations, and the coordinates for some items
*    on the Display screen are used as peek()
*    locations.  Changing either will require
*    substantially reworking the program.

**		    Main procedure 		

SAVE SCREEN TO Lc_screen                       		 
CLEAR
@ 11,23 TO 14,56 DOUBLE
@ 12,25 SAY "COLORSET Color Change Utility "
@ 13,25 SAY "Copyright (c) 1989 Jay Parsons"
Endsec=val(right(time(),2))+Banner

*         Test for video display values using VINFO.BIN  

*    See notes in VINFO.ASM listing for details.

LOAD vinfo
Arg1=chr(2)
STORE " " TO Arg2,Arg3,Arg4,Arg5,Arg6,Arg7
CALL vinfo with Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7
Vchars=asc(arg1)    && characters per screen line
Vmode=asc(arg2)     && video mode
Vpage=asc(Arg3)     && video page
Vtype=Arg4          && display type letter
Vcode=asc(Arg5)     && combination code
Vfont=asc(Arg6)     && character height in pixels
Vlastrow=asc(Arg7)  && last row number

*         Quit on errors relating to display type found

Err=.F.
IF Vtype $ "?P0"
  @ 16,14 SAY "Unknown display type.  ";
    +"Unable to set colors.  Sorry."
  Err=.T.
ENDIF
IF Vtype="M"
  @ 16,8 SAY "This program requires ";
    +"color-compatible display adapter.  Sorry."
  Err=.T.
ENDIF  
IF Vmode # 3 
  @ 16,4 SAY "Cannot set colors while suppressed.  ";
    +"Please SET DISPLAY TO a color mode."
  Err=.T.
ENDIF     
IF Err
  @ 22,0
  WAIT
  RELEASE MODULE vinfo   
  RELEASE POPUPS init,mems,csave
  RESTORE SCREEN FROM Lc_screen
  RETURN
ENDIF 

*         Save entry settings, then change as required
*    Note: This program does NOT preserve the .dbf
*    file in use in the current work area, if any.

Lc_status=set("STATUS")
Lc_head=set("HEADING")
Lc_score=set("SCOREBOARD")

*     Convert display type to one recognized by dBASE

Lc_disp=iif(vtype $ "VE","EGA" ;
   +iif(vlastrow=24,"25","43"),"COLOR")
LOAD peek

*     Keep banner for 'Banner' seconds 

DO WHILE .T.
  Nowsec=val(right(time(),2));
    +iif(Endsec>60-Banner,60,0)
  IF Nowsec>Endsec
    EXIT
  ENDIF
ENDDO

IF Lc_disp="EGA43"
  SET DISPLAY TO EGA25
ENDIF   
SET STATUS ON
Lc_mess=ltrim(trim(charpeek(24,0,80)))
SET SCOREBOARD OFF
SET HEADING ON

*	        Initialize data and get starting colors

Mfile=""            && filespec for save file
Ffile=""            && filespec of Config.db
Dfile=dbf()         && data file
Inuse=("" # Dfile)  && .T. if there is a file open
DECLARE O[7],C[8]   && attributes for color areas
STORE 0 TO O[1],O[2],O[3],O[4],O[5],O[6],O[7],Row,Clm
STORE " " TO C_norm,C_mess,C_titl,C_box,C_high,;
  C_info,C_fiel,Cnormal,Cfields
DO colorshow             && show colors and peek them
IF Speedup
  LOAD showit
ENDIF

*         Main loop 

DO WHILE .T.
  DO paint              && paint the main screen
  DO getchg             && get changes to colors
  IF Clm # Saveclm      && if told to cancel changes
    DO oldcols          && restore and done
    EXIT
  ENDIF
  DO update             && else update dBASE colors
  CLEAR
  @ 1,26 SAY "COLORSET Save Options Screen"
  ACTIVATE POPUP csave  && get save option
  DO CASE
    CASE bar()=5
      DO colorshow      && start over, no peeks
      LOOP
    CASE bar()=6
      DO Oldcols        && restore and done
  ENDCASE               && otherwise, just done
  EXIT
ENDDO
  
*         Restore message, status, header & display
*         and all done

RELEASE MODULE peek      && Ver 1.0 does not allow
RELEASE MODULE vinfo     && these lines to be combined
IF Speedup
  RELEASE MODULE showit
ENDIF
RELEASE POPUPS init,mems,csave 
IF .NOT. Inuse
  USE
ENDIF  
IF "" # Lc_mess
  SET MESSAGE TO Lc_mess
ENDIF  
IF Lc_status="ON"
  SET STATUS ON
ENDIF
IF Lc_head="OFF"
  SET HEADING OFF
ENDIF    
IF Lc_score="ON"
  SET SCOREBOARD ON
ENDIF
IF Lc_disp="EGA43"
  SET DISPLAY TO EGA43
ENDIF
RESTORE SCREEN FROM Lc_screen
RETURN

**        End of main program

**        Major subroutines

*    Find and show colors in use - uses Peek.bin

PROCEDURE colorshow
PRIVATE X,Fline,Heading,W,Z
CLEAR
SET STATUS ON       && first because clears screen
SET MESSAGE TO "This message and the unselected ";
  +"bars of the menu use MESSAGE colors."
@ 1,25 SAY "COLORSET Color Display Screen" 
@ 3,3 SAY "This text uses NORMAL colors.  The box ";
  +"and menu borders use BOX colors,"
@ 4,3 SAY "and the heading within the larger box at ";
  +"lower right uses TITLES colors."
@ 5,3 SAY "The selected bar of the menu at screen ";
  +"center is in HIGHLIGHTS colors,"
@ 6,3 SAY "while the status line above the message ";
  +"at bottom uses INFORMATION colors."
Fline="This line is in FIELDS colors. Please choose ";
  +"the colors to start from:"
@ 8,5 GET Fline
DEFINE WINDOW W FROM 11,26 TO 19,78
ACTIVATE WINDOW W
IF "" = Dfile       && if no file in use
  DIR               && list the .DBF files
  Dfile=charpeek(14,27,8)   && pick the last
  Dfile=iif("." $ Dfile,left(Dfile,;
    +at(".",Dfile)-1),trim(Dfile))+".DBF"
  IF file(Dfile)
    USE (Dfile)     && and use it
  ELSE
    DO openedit     && or create the edit file
    Dfile=(Editfile)
  ENDIF
ENDIF
Z=(Reccount()=0)
IF Z
  APPEND BLANK
ENDIF
Fieldname=field(1)  && display an expression
Heading=iif(type(Fieldname)="C",left(&Fieldname,40),;
   &Fieldname)
@ 6,0               && that won't overflow one line
DISPLAY Heading
ACTIVATE SCREEN
@ 17,1 SAY "These are TITLES colors-->"
ACTIVATE POPUP init
IF O[1]=0
  O[1]=attrpeek(3,0)   && normal text
  O[2]=attrpeek(24,39) && messages
  O[3]=attrpeek(17,27) && titles
  O[4]=attrpeek(11,26) && boxes
  O[6]=attrpeek(22,0)  && status line & clock
  O[7]=attrpeek(8,6)   && fields
ENDIF
SET STATUS OFF
IF C_norm # " "   && if we have some colors
  C[1]=attrcode(C_norm)
  C[2]=attrcode(C_mess)
  C[3]=attrcode(C_titl)
  C[4]=attrcode(C_box)
  C[5]=attrcode(C_high)
  C[6]=attrcode(C_info)
  C[7]=attrcode(C_fiel)
ELSE
  X=1
  DO WHILE X<8
    C[X]=O[X]
    X=X+1
  ENDDO
  DO ccodes
ENDIF
Cnormal=mod(C[1],128)    && normal, without blink
Cfields=mod(C[7],128)    && fields, ditto
C[8]=Cnormal             && color for screen line 20
IF Z
  DELETE ALL
  PACK
ENDIF
CLEAR GETS
RELEASE WINDOW W
RETURN

*    Paint main screen

PROCEDURE paint
PRIVATE I
SET STATUS OFF
SET MESSAGE TO 
CLEAR
@ 1,25 SAY "COLORSET Color Selection Screen" 
@ 3,Labelclm+Block/2-6 SAY "Display Area" 
@ 3,Showclm+Block/2-5 SAY "Appearance" 
@ 3,Fgclm+Block/2-8 SAY "Foreground/Bright"
@ 3,Bgclm+Block/2-8 SAY "Background/Blink"
@ 4,Labelclm SAY replicate("-",Block)
@ 4,Showclm SAY replicate("-",Block)
@ 4,Fgclm SAY replicate("-",Block)
@ 4,Bgclm SAY replicate("-",Block) 
@ 6,Labelclm SAY  "Normal text" 
@ 8,Labelclm SAY "Messages" 
@ 10,Labelclm SAY "Titles" 
@ 12,Labelclm SAY "Box" 
@ 14,Labelclm SAY "Highlights" 
@ 16,Labelclm SAY "Information" 
@ 18,Labelclm SAY "Fields" 
@ 20,Labelclm+4 SAY "All finished -"
@ 22,40-len(Navmess1)/2 SAY Navmess1
@ 23,40-len(Navmess2)/2 SAY Navmess2 
Row=6
DO WHILE Row<20
  I=Row/2-2
  Clm=Showclm
  DO show WITH cname(C[I]),C[I]    && actual colors
  Clm=Fgclm
  DO show WITH cname(C[I]),Cnormal && foreground/normal
  Clm=Bgclm
  DO show WITH cname(C[I]),Cnormal && background/normal
  Row=Row+2
ENDDO
@ 20,Saveclm SAY Savemess
@ 20,iif(Saveclm=Fgclm,Bgclm,Fgclm) SAY Canmess
Row=6                         && start at upper left
Clm=Fgclm
I=1
DO show WITH cname(C[I]),Cfields   && highlight a color
RETURN

*    Color selection loop - Get and display
*    changes of color

PROCEDURE getchg
PRIVATE I,Oclm,X,Nrow,Nclm
DO WHILE .T.
  @ Row,Clm say ""
  X=inkey(0)
  DO CASE
    CASE X=Q .AND. Row=20     && finished with changes
      EXIT
    CASE X=L .OR. X=R    && fore/back change of ground
      Nclm=iif(Clm=Bgclm,Fgclm,Bgclm)
      Nrow=Row
      DO switch
    CASE X=Dn .OR. X=Up  && up or down 2, limits 6 & 20
      Nrow=mod(10+Row+iif(X=Up,-2,2),16)+6
      Nclm=Clm
      DO switch
    CASE (X=Ucolor .OR. X=Dcolor) .AND. Row<20 
      I=Row/2-2          && adjust color by 1 bit
      Oclm=Clm
      IF Clm=Fgclm
        C[I]=16*int(C[I]/16)+mod(16+C[I];
          +iif(X=Ucolor,1,-1),16)
      ELSE
        C[I]=mod(C[I],16)+16*mod(16+int(C[I]/16);
          +iif(X=Ucolor,1,-1),16)
      ENDIF
      Clm=Showclm
      DO show WITH cname(C[I]),C[I]     && actual
      Clm=Oclm
      DO show WITH cname(C[I]),Cfields  && highlight
  ENDCASE
ENDDO      
SET MESSAGE TO
RETURN

**        Popup handlers

*    Select colors to start with.
*    Peek.bin must be loaded if O[5]=0.

PROCEDURE init_pop                             
IF O[5]=0
  O[5]=attrpeek(bar()+10,19)  && highlight colors
ENDIF
DO CASE
  CASE bar()=1
    IF Reconfig               && look in Config.db
      DO readcfig
    ELSE
      RESTORE FROM &dflfile ADDITIVE
    ENDIF Reconfig or not  
  CASE bar()=2
    ON SELECTION POPUP mems DO mr_pop 
    ACTIVATE POPUP mems
ENDCASE                       && no action for bar()=3
DEACTIVATE POPUP
RETURN

PROCEDURE mr_pop
IF file(prompt())
  RESTORE FROM prompt() ADDITIVE
ENDIF
DEACTIVATE POPUP
RETURN

*    Save colors for future

PROCEDURE cs_pop
PRIVATE Colormem,W
DO CASE
  CASE bar()=1
    IF Reconfig
      DO Savecfig
    ELSE    
      SAVE ALL LIKE C_* TO &dflfile
    ENDIF  
  CASE bar()=2
    ON SELECTION POPUP mems DO ms_pop 
    ACTIVATE POPUP mems
  CASE bar()=3
    Colormem=space(78)
    DEFINE WINDOW W FROM 11,0 TO 14,79
    ACTIVATE Window W
    @ 0,0 SAY "Enter name (including path if needed) ";
      +"of ."+Memext+" file:"
    @ 1,0 GET Colormem 
    READ
    Colormem=iif("." $ Colormem,left(Colormem,;
       at(".",Colormem)-1),trim(Colormem))+"."+Memext
    SAVE ALL LIKE C_* TO &Colormem
    RELEASE WINDOW W
ENDCASE                  && no action for bar()>3
DEACTIVATE POPUP
RETURN         

PROCEDURE ms_pop
SAVE ALL LIKE C_* TO prompt()
DEACTIVATE POPUP
RETURN

**        Miscellaneous functions and procedures
**        (in alphabetical order)

*    Peek attribute byte at row and column of current
*    video page

FUNCTION attrpeek
PARAMETERS Row,Col
PRIVATE Vpage
Vpage=" "
CALL vinfo WITH chr(1)," ",Vpage
RETURN asc(call("peek",;
   vidaddr(Row,Col,asc(Vpage),1),1))

*    Convert dBASE color code to attribute value

FUNCTION attrcode
PARAMETERS Ccode
PRIVATE Attr,Fore,Back,Slash
Slash=at("/",Ccode)
Fore=iif(Slash=0,Ccode,left(Ccode,Slash-1))
Back=iif(Slash=0,"N",substr(Ccode,Slash+1))
Attr=0+iif("B" $ Fore,1,0)+iif("G" $ Fore,2,0);
  +iif("R" $ Fore,4,0)+iif("W" $ Fore,7,0);
  +iif("+" $ Ccode,8,0)
Attr=Attr+iif("B" $ Back,16,0);
  +iif("G" $ Back,32,0)+iif("R" $ Back,64,0);
  +iif("W" $ Back,112,0)+iif("*" $ Ccode,128,0)
RETURN iif("X" $ Fore,17*mod(int(Attr/16),8),;
  iif("X" $ Back,17*mod(Attr,8),Attr))

*    Convert attribute array to dBASE color codes

PROCEDURE ccodes
C_norm=cval(C[1])
C_mess=cval(C[2])
C_titl=cval(C[3])
C_box =cval(C[4])
C_high=cval(C[5])
C_info=cval(C[6])
C_fiel=cval(C[7])
RETURN

*    Peek N characters starting at a row and column
*    of current video page.
*    N must be less than 126 since the address argument
*    must, in order to hold the characters and
*    attributes returned, be a string of length 2*N,
*    and the address itself can vary from 6-10 bytes.

FUNCTION charpeek
PARAMETERS Row,Col,N
PRIVATE X,X1,C,Vpage,Addr
Vpage=" "
CALL vinfo WITH chr(1)," ",Vpage
Addr=vidaddr(Row,Col,asc(Vpage),0)+space(2*N-6)
X=call("peek",Addr,2*N)
X1=space(N)
C=1
DO WHILE C<=N       && remove attribute bytes
  X1=stuff(X1,C,1,substr(X,2*C-1,1))
  C=C+1
ENDDO
RETURN X1      

*    Construct name of a color.  If in background
*    column, use the name of the background of the
*    color, else of the foreground.  Returns name
*    centered in a string of len()=Block.

FUNCTION cname
PARAMETERS C
PRIVATE Colr,Name
Colr=iif(Clm=Bgclm,mod(int(C/16),8),mod(C,16))
Name=trim(substr("Black  Blue   Green  Cyan   ";
  +"Red    MagentaBrown  White  ",mod(Colr,8)*7+1,7))
IF Clm # Bgclm
  DO CASE
    CASE Colr=8
      Name="Gray"
    CASE Colr=14
      Name="Yellow"
    CASE Colr>7
      Name="Bright "+Name
  ENDCASE
ELSE
  Name=iif(C>127,Name+"+Blinking",Name)
ENDIF  
RETURN substr(space(Block/2)+Name+space(Block/2),;
  int((len(Name)+3)/2),Block)

*    Convert attribute value to dBASE color code

FUNCTION cval
PARAMETERS Attr
PRIVATE Colrs
Colrs="N B G BGR RBGRW "
RETURN trim(substr(Colrs,mod(Attr,8)*2+1,2));
  +iif(mod(int(Attr/8),2)>0,"+","");
  +iif(Attr>127,"*","")+"/";
  +trim(substr(Colrs,mod(int(Attr/16),8)*2+1,2))

*    Restore original colors from the O[] array.

PROCEDURE oldcols
X=1
DO WHILE X<8
  C[X]=O[X]
  X=X+1
ENDDO
DO update
RETURN

*    Open or create the edit file

PROCEDURE openedit
PRIVATE F
Inuse=.F.           && set flag to close it later
IF .NOT. file(Editfile)  && and if it doesn't exist,
  DECLARE F[1,4]
  F[1,1]="LINE"          && create it with this field.
  F[1,2]="C"
  F[1,3]=77
  F[1,4]=0
  DO make_dbf WITH (Editfile),"F",1
ENDIF
USE (Editfile)  
RETURN

*    Show characters in colors of C[I] on given
*    background.  If not at Showclm, change
*    foreground color as needed for contrast.
*    If Speedup is .T., Showit.bin must be in memory.

PROCEDURE show
PARAMETERS Chars,Back
PRIVATE Blink,Bg,Fg,Colr 
Blink=iif(C[I]>127 .AND. Clm=Showclm,1,0)
Bg=mod(int(Back/16),8)
Fg=iif(Clm=Bgclm,mod(int(C[I]/16),8),mod(C[I],16))
Fg=iif(Clm<>Showclm .AND.;
   mod(Fg,8)=Bg,iif(Bg=0,7,0),Fg)
Colr=cval(128*Blink+16*Bg+Fg)
IF Speedup
  @ Row,Clm SAY ""
  CALL showit WITH Chars,chr(attrcode(Colr))
ELSE  
  SET COLOR OF NORMAL TO &Colr
  @ Row,Clm SAY Chars 
  Colr=cval(Cnormal)
  SET COLOR OF NORMAL TO &Colr
ENDIF
RETURN

*    Move cursor to selected field and show it on
*    field background, old field on normal background.

PROCEDURE switch
PRIVATE I
I=Row/2-2
DO show WITH iif(Row<20,cname(C[I]),;
  iif(Clm=Saveclm,Savemess,Canmess)),Cnormal
Row=Nrow
Clm=Nclm
I=Row/2-2
DO show WITH iif(Row<20,cname(C[I]),;
  iif(Clm=Saveclm,Savemess,Canmess)),Cfields
SET MESSAGE TO iif(Row<20,"",;
  iif(Clm=Saveclm,Savemes2,Canmes2))
RETURN

*    Update the dBASE color settings from the C[] array

PROCEDURE update
DO ccodes
IF Setborder
  C_back=",,"+substr(C_norm,at("/",C_norm)+1)
  SET COLOR TO &C_back
ENDIF
DO cchange
RETURN

*    Calculate video display address.
*    47104 is B800h, segment of CGA memory.
*    256 is page size in paragraphs, 4096 bytes/16.
*    Assumes 80 characters/row.
*    The fourth argument is 0 for the character,
*    1 for the attribute.

FUNCTION vidaddr
PARAMETERS Row,Col,Page,Which
RETURN dectohex(47104+Page*256)+":";
   +dectohex(2*(80*Row+Col)+Which)
                                                      
******************************************************
*   The remaining portions of the program are called,
*   and need be included, only if Reconfig may be .T.
******************************************************

*    Read colors from Config.db, or use system defaults
*    for areas not set in Config.db

PROCEDURE readcfig
PRIVATE Fieldname,Value,Norm,Enhance,Carea
C_norm="W+/B"  && W/N    && Assume system defaults.
C_mess="W/N"   && W+/N   && The comment values will be
C_titl="W/B"   && W/N    && the system defaults if
C_box ="GR+/BG"&& N/W    && dBSETUP has not been used
C_high="GR+/BG"&& N+/W   && to specify a color display,
C_info="B/W"   && N/W    && even if Config.db contains
C_fiel="N/BG"  && N/W    && "DISPLAY=COLOR".
DO openedit
Fieldname=field(1)
DO loadcfig
IF "" # Ffile
  SCAN FOR left(&Fieldname,4)="COLO"
    Value=substr(&Fieldname,at("=",&Fieldname)+1)
    DO CASE
      CASE "ON" $ Value .OR. "OFF" $ Value && no action
      CASE "OF" $ &Fieldname
        Carea="C_"+left(ltrim(substr(&Fieldname,;
          at("OF",&Fieldname)+2)),4)
        Carea=iif(right(Carea,1)="=",;
          left(Carea,5),Carea)
        STORE ltrim(Value) TO &Carea
      OTHERWISE
        Norm=iif("," $ Value,;
          left(Value,at(",",Value)-1),;
          Value)
        IF "" # Norm
          STORE ltrim(Norm) TO C_norm,C_mess,C_titl
        ENDIF
        Enhance=iif("," $ Value,substr(Value,;
          at(",",Value)+1),"")
        IF "" # Enhance
          STORE ltrim(Enhance) TO C_box,C_high,;
            C_info,C_fiel
        ENDIF
    ENDCASE
  ENDSCAN
ENDIF
RETURN

*    Save colors to Config.db found, or create it

PROCEDURE savecfig
PRIVATE Fieldname,Value
DO openedit
Fieldname=field(1)
IF ""=Ffile         && if Config.db not yet loaded
  DO loadcfig
ENDIF
IF "" # Ffile
  SCAN FOR left(&Fieldname,4)="COLO"
    Value=substr(&Fieldname,at("=",&Fieldname)+1)
    IF .NOT.  ("ON" $ Value .OR. "OFF" $ Value)
      DELETE NEXT 1
      IF .NOT. (Setborder .OR. "OF" $ Value)
        Value=iif("," $ Value,substr(Value,;
          at(",",Value)+1),"")  && skip norm
        Value=iif("," $ Value,substr(Value,;
          at(",",Value)+1),"")  && and enhance
        IF "" # Value    && save overscan if any set
          Value=",,"+iif("," $ Value,;
            left(Value,at(",",Value)-1),Value)
          APPEND BLANK
          REPLACE &Fieldname WITH ;
             "COLOR                = "+Value
        ENDIF overscan color set 
      ENDIF a simple "color=" command without Setborder
    ENDIF not "on" or "off"
  ENDSCAN                
ELSE
  Ffile="CONFIG.DB"
ENDIF
IF Setborder
  APPEND BLANK
  REPLACE &Fieldname WITH ;
     "COLOR                = ,,";
     +substr(C_norm,at("/",C_norm)+1)
ENDIF
APPEND BLANK    
REPLACE &Fieldname WITH ;
  "COLOR OF NORMAL      = "+C_norm
APPEND BLANK
REPLACE &Fieldname WITH ;
  "COLOR OF MESSAGES    = "+C_mess
APPEND BLANK    
REPLACE &Fieldname WITH ;
  "COLOR OF TITLES      = "+C_titl
APPEND BLANK
REPLACE &Fieldname WITH ;
  "COLOR OF BOX         = "+C_box
APPEND BLANK    
REPLACE &Fieldname WITH ;
  "COLOR OF HIGHLIGHT   = "+C_high
APPEND BLANK
REPLACE &Fieldname WITH ;
  "COLOR OF INFORMATION = "+C_info
APPEND BLANK    
REPLACE &Fieldname WITH ;
  "COLOR OF FIELDS      = "+C_fiel
COPY TO (Ffile) SDF      && trouble if this fails
RETURN

*    Find Config.db and load it into the active .dbf.

PROCEDURE loadcfig
ZAP
Ffile=dosfindf("CONFIG.DB")
IF "" # Ffile
  APPEND FROM (Ffile) SDF
  REPLACE ALL &Fieldname WITH ltrim(upper(&Fieldname))
ENDIF
RETURN
