;	vinfo.asm - video display information routine.

;       Unless otherwise stated, all numerical
;       parameters are passed and returned as bytes
;       with binary values - in dBASE as chr(argument).
;       dBASE can read the value returned with asc().

;	Called with three or seven arguments.

;       Expects arguments in dBASE IV format and
;       locations, i.e.:
;               ES:DI points to table of far pointers
;                to arguments.
;		CX holds number of arguments.	

;	****  Service 1  ****

;       Called with first parameter=chr(1), two others:

;       Returns following in place of the arguments
;        from BIOS service 0Fh:

;       pmtr1:  number of characters to a screen line
;               in current mode, usually 80.
;	pmtr2:	Current video mode.
;	pmtr3:	Current video page.	

;       Service 1 can be called with extra parameters,
;       but they will not be used or affected.  It is
;       faster than a call to service 2, but by an
;       amount of time usually negligible.

;	****  Service 2  ****

;       Called with first parameter=chr(2), six others:

;	Returns same values as service 1 plus:

;       pmtr4:  code among following characters
;               for display configuration:
;               0=no display, M=MDA, C=CGA, E=EGA,
;               P=PGC, V=VGA, 2=MCGA, ?=unknown.
;       pmtr5:  Display combination code returned by
;               Int 10h service 1Ah in BL--
;               FFh if call fails.
;       pmtr6:  Height of characters in rows of pixels
;       pmtr7:  Number of last row on screen
;               (Last two are garbage if #4 is 0 or ?


;       This program must be assembled and linked, then
;       converted to a binary image file by EXE2BIN.
;       It cannot be converted to a .COM file.

;		Definitions of services supported

brief   equ 1
long    equ 2

        .model tiny
        code segment
        assume cs:code

vinfo   proc far
        jmp short start

vtable  db '0MC?EEPVV??22'      ; codes for service 1Ah

start:  jcxz erret2             ; must be arguments
        lds si,es:[di]          ; point to first parm
        cmp byte ptr [si],long  ; long routine?
        jz getl                 ; then jump

        cmp cx,3                ; enough parameters?
        jb erret                ; jump if not
        mov ah,0Fh              ; get mode
        int 10h                 ; call BIOS
vsave:  lds si,es:[di]          ; point to first parm
        mov [si],ah             ; save chars-per-line
        lds si,es:[di]+4        ; point to second
        mov [si],al             ; save video mode in it
        lds si,es:[di]+8        ; point to third
        mov [si],bh             ; it gets the page
vidret: ret                     ; and done

erret:  lds si,es:[di]          ; replace #1 with '?'
        mov byte ptr [si],'?'
erret2: ret                     ; and that's all

;	Get long video display information.

getl:   cmp cx,7                ; enough parameters?
        jb erret
        mov ax,1A00h            ; display combo code
        int 10h
        lds si,es:[di]+16       ; save to #5
        mov [si],bl
        cmp al,1Ah              ; did it work?
        mov dh,'?'              ; flag for no
        jne No_vga              ; jump on failure
        cmp bl,0                ; if bl not 0,
        jnz getact              ; it is active display
        mov bl,bh               ; otherwise bh is

getact: cmp bl,0Dh
        jae gmode               ; can't be above 0Ch
        mov al,bl
        mov bx,offset vtable    ; point to table
        xlat cs:vtable          ; look it up
        mov dh,al               ; save in dh
        jmp short gmode

No_vga: mov byte ptr [si],0FFh  ; failure code to #5
        mov ah,12h              ; get EGA info
        mov bl,10h
        int 10h
        mov dh,'E'
        cmp bl,10h              ; did this change?
        jnz gmode               ; then we have EGA
        mov dh,' '              ; else don't know yet

gmode:  xor cl,cl               ; blank these if so
        mov dl,cl
        cmp dh,'?'              ; failure?
        jz gsave
        mov ah,0Fh
        int 10h                 ; get the mode
        cmp dh,' '              ; already got type?
        jnz gotm
        mov dl,24               ; 24 usually last row
        mov cl,14               ; 14 is MDA char height
        cmp al,7                ; mono mode?
        mov dh,'M'              ; then must be MDA
        jz gsave
        mov dh,'C'              ; else CGA
        mov cl,8                ; with 8-char height
        jmp short gsave

gotm:   push ax                 ; save mode
        push bx
        push es                 ; and segment
        mov ax,1130h            ; get font information
        xor bx,bx               ; default font
        int 10h                 ; call BIOS
        pop es
        pop bx
        pop ax
gsave:  lds si,es:[di]+12       ; fourth parameter
        mov [si],dh             ; display type letter
        lds si,es:[di]+20       ; sixth parameter
        mov [si],cl             ; character height
        lds si,es:[di]+24       ; seventh and last
        mov [si],dl
        jmp vsave

vinfo   endp
code    ends
        end
